// Generated by delombok at Mon Oct 04 14:27:44 EDT 2021
package gov.cms.fiss.pricers.irf.api;

import gov.cms.fiss.pricers.common.util.LocalDateUtils;
import io.dropwizard.validation.ValidationMethod;
import io.swagger.v3.oas.annotations.extensions.Extension;
import io.swagger.v3.oas.annotations.extensions.ExtensionProperty;
import io.swagger.v3.oas.annotations.media.Schema;
import java.math.BigDecimal;
import javax.validation.constraints.NotEmpty;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;

@Schema(description = "The billing information associated with the claim.", extensions = {@Extension(name = "cobol-declaration", properties = {@ExtensionProperty(name = "BILL-NEW-DATA", value = "<structure>")})})
public class BillingRecord {
  @Schema(description = "The national provider identifier (NPI).", example = "1649259600", extensions = {@Extension(name = "cobol-declaration", properties = {@ExtensionProperty(name = "B-NPI10", value = "B-NPI8 PIC X(08)\nB-NPI-FILLER PIC X(02)", parseValue = true)})})
  private String npi;
  @Schema(description = "The provider number.", example = "523027", extensions = {@Extension(name = "cobol-declaration", properties = {@ExtensionProperty(name = "B-PROVIDER-NO", value = "PIC X(06)")})})
  private String providerNumber;
  @Schema(description = "The patient status.", example = "02", extensions = {@Extension(name = "cobol-declaration", properties = {@ExtensionProperty(name = "B-PATIENT-STATUS", value = "PIC X(02)")})})
  @NotNull
  @NotEmpty
  private String patientStatus;
  // The minimum length of a cmgCode is 5, but this is handled as a 200 with
  // ResultCode.CMG_ON_CLAIM_NOT_IN_TABLE_54. See editCmgCode() of each IrfPricer class.
  @NotNull
  @Size(max = 5)
  @Schema(description = "The case-mix group (CMG) code.", example = "C2004", extensions = {@Extension(name = "cobol-declaration", properties = {@ExtensionProperty(name = "B-CMG-CODE", value = "PIC X(05)")})})
  private String cmgCode;
  @Schema(description = "The length of stay (LOS).", example = "5", extensions = {@Extension(name = "cobol-declaration", properties = {@ExtensionProperty(name = "B-LOS", value = "PIC 9(03)")})})
  private int lengthOfStay;
  @Schema(description = "The covered days.", example = "5", extensions = {@Extension(name = "cobol-declaration", properties = {@ExtensionProperty(name = "B-COV-DAYS", value = "PIC 9(03)")})})
  private int coveredDays;
  @Schema(description = "The lifetime reserve (LTR) days.", example = "0", extensions = {@Extension(name = "cobol-declaration", properties = {@ExtensionProperty(name = "B-LTR-DAYS", value = "PIC 9(02)")})})
  private int lifetimeReserveDays;
  @Schema(description = "The special pay indicator.", example = "1", extensions = {@Extension(name = "cobol-declaration", properties = {@ExtensionProperty(name = "B-SPEC-PAY-IND", value = "PIC X(01)")})})
  @NotNull
  private String specialPayIndicator;
  @Schema(description = "The discharge date.", example = "20170415", extensions = {@Extension(name = "cobol-declaration", properties = {@ExtensionProperty(name = "B-DISCHARGE-DATE", value = "B-DISCHG-CC PIC 9(02)\n" + "B-DISCHG-YY PIC 9(02)\n" + "B-DISCHG-MM PIC 9(02)\n" + "B-DISCHG-DD PIC 9(02)", parseValue = true)})})
  @NotNull
  private String dischargeDate;
  @Schema(description = "The covered charges.", example = "10212.75", extensions = {@Extension(name = "cobol-declaration", properties = {@ExtensionProperty(name = "B-COV-CHARGES", value = "PIC 9(07)V9(02)")})})
  @NotNull
  private BigDecimal coveredCharges;

  @ValidationMethod
  public boolean isDischargeDate() {
    // Null check is performed elsewhere so return if null
    if (dischargeDate == null) {
      return true;
    }
    return LocalDateUtils.fromYearFormat(dischargeDate) != null;
  }

  @java.lang.SuppressWarnings("all")
  @lombok.Generated
  public BillingRecord(final String npi, final String providerNumber, final String patientStatus, final String cmgCode, final int lengthOfStay, final int coveredDays, final int lifetimeReserveDays, final String specialPayIndicator, final String dischargeDate, final BigDecimal coveredCharges) {
    this.npi = npi;
    this.providerNumber = providerNumber;
    this.patientStatus = patientStatus;
    this.cmgCode = cmgCode;
    this.lengthOfStay = lengthOfStay;
    this.coveredDays = coveredDays;
    this.lifetimeReserveDays = lifetimeReserveDays;
    this.specialPayIndicator = specialPayIndicator;
    this.dischargeDate = dischargeDate;
    this.coveredCharges = coveredCharges;
  }

  @java.lang.SuppressWarnings("all")
  @lombok.Generated
  public String getNpi() {
    return this.npi;
  }

  @java.lang.SuppressWarnings("all")
  @lombok.Generated
  public String getProviderNumber() {
    return this.providerNumber;
  }

  @java.lang.SuppressWarnings("all")
  @lombok.Generated
  public String getPatientStatus() {
    return this.patientStatus;
  }

  @java.lang.SuppressWarnings("all")
  @lombok.Generated
  public String getCmgCode() {
    return this.cmgCode;
  }

  @java.lang.SuppressWarnings("all")
  @lombok.Generated
  public int getLengthOfStay() {
    return this.lengthOfStay;
  }

  @java.lang.SuppressWarnings("all")
  @lombok.Generated
  public int getCoveredDays() {
    return this.coveredDays;
  }

  @java.lang.SuppressWarnings("all")
  @lombok.Generated
  public int getLifetimeReserveDays() {
    return this.lifetimeReserveDays;
  }

  @java.lang.SuppressWarnings("all")
  @lombok.Generated
  public String getSpecialPayIndicator() {
    return this.specialPayIndicator;
  }

  @java.lang.SuppressWarnings("all")
  @lombok.Generated
  public String getDischargeDate() {
    return this.dischargeDate;
  }

  @java.lang.SuppressWarnings("all")
  @lombok.Generated
  public BigDecimal getCoveredCharges() {
    return this.coveredCharges;
  }

  @java.lang.SuppressWarnings("all")
  @lombok.Generated
  public void setNpi(final String npi) {
    this.npi = npi;
  }

  @java.lang.SuppressWarnings("all")
  @lombok.Generated
  public void setProviderNumber(final String providerNumber) {
    this.providerNumber = providerNumber;
  }

  @java.lang.SuppressWarnings("all")
  @lombok.Generated
  public void setPatientStatus(final String patientStatus) {
    this.patientStatus = patientStatus;
  }

  @java.lang.SuppressWarnings("all")
  @lombok.Generated
  public void setCmgCode(final String cmgCode) {
    this.cmgCode = cmgCode;
  }

  @java.lang.SuppressWarnings("all")
  @lombok.Generated
  public void setLengthOfStay(final int lengthOfStay) {
    this.lengthOfStay = lengthOfStay;
  }

  @java.lang.SuppressWarnings("all")
  @lombok.Generated
  public void setCoveredDays(final int coveredDays) {
    this.coveredDays = coveredDays;
  }

  @java.lang.SuppressWarnings("all")
  @lombok.Generated
  public void setLifetimeReserveDays(final int lifetimeReserveDays) {
    this.lifetimeReserveDays = lifetimeReserveDays;
  }

  @java.lang.SuppressWarnings("all")
  @lombok.Generated
  public void setSpecialPayIndicator(final String specialPayIndicator) {
    this.specialPayIndicator = specialPayIndicator;
  }

  @java.lang.SuppressWarnings("all")
  @lombok.Generated
  public void setDischargeDate(final String dischargeDate) {
    this.dischargeDate = dischargeDate;
  }

  @java.lang.SuppressWarnings("all")
  @lombok.Generated
  public void setCoveredCharges(final BigDecimal coveredCharges) {
    this.coveredCharges = coveredCharges;
  }

  @java.lang.Override
  @java.lang.SuppressWarnings("all")
  @lombok.Generated
  public boolean equals(final java.lang.Object o) {
    if (o == this) return true;
    if (!(o instanceof BillingRecord)) return false;
    final BillingRecord other = (BillingRecord) o;
    if (!other.canEqual((java.lang.Object) this)) return false;
    if (this.getLengthOfStay() != other.getLengthOfStay()) return false;
    if (this.getCoveredDays() != other.getCoveredDays()) return false;
    if (this.getLifetimeReserveDays() != other.getLifetimeReserveDays()) return false;
    final java.lang.Object this$npi = this.getNpi();
    final java.lang.Object other$npi = other.getNpi();
    if (this$npi == null ? other$npi != null : !this$npi.equals(other$npi)) return false;
    final java.lang.Object this$providerNumber = this.getProviderNumber();
    final java.lang.Object other$providerNumber = other.getProviderNumber();
    if (this$providerNumber == null ? other$providerNumber != null : !this$providerNumber.equals(other$providerNumber)) return false;
    final java.lang.Object this$patientStatus = this.getPatientStatus();
    final java.lang.Object other$patientStatus = other.getPatientStatus();
    if (this$patientStatus == null ? other$patientStatus != null : !this$patientStatus.equals(other$patientStatus)) return false;
    final java.lang.Object this$cmgCode = this.getCmgCode();
    final java.lang.Object other$cmgCode = other.getCmgCode();
    if (this$cmgCode == null ? other$cmgCode != null : !this$cmgCode.equals(other$cmgCode)) return false;
    final java.lang.Object this$specialPayIndicator = this.getSpecialPayIndicator();
    final java.lang.Object other$specialPayIndicator = other.getSpecialPayIndicator();
    if (this$specialPayIndicator == null ? other$specialPayIndicator != null : !this$specialPayIndicator.equals(other$specialPayIndicator)) return false;
    final java.lang.Object this$dischargeDate = this.getDischargeDate();
    final java.lang.Object other$dischargeDate = other.getDischargeDate();
    if (this$dischargeDate == null ? other$dischargeDate != null : !this$dischargeDate.equals(other$dischargeDate)) return false;
    final java.lang.Object this$coveredCharges = this.getCoveredCharges();
    final java.lang.Object other$coveredCharges = other.getCoveredCharges();
    if (this$coveredCharges == null ? other$coveredCharges != null : !this$coveredCharges.equals(other$coveredCharges)) return false;
    return true;
  }

  @java.lang.SuppressWarnings("all")
  @lombok.Generated
  protected boolean canEqual(final java.lang.Object other) {
    return other instanceof BillingRecord;
  }

  @java.lang.Override
  @java.lang.SuppressWarnings("all")
  @lombok.Generated
  public int hashCode() {
    final int PRIME = 59;
    int result = 1;
    result = result * PRIME + this.getLengthOfStay();
    result = result * PRIME + this.getCoveredDays();
    result = result * PRIME + this.getLifetimeReserveDays();
    final java.lang.Object $npi = this.getNpi();
    result = result * PRIME + ($npi == null ? 43 : $npi.hashCode());
    final java.lang.Object $providerNumber = this.getProviderNumber();
    result = result * PRIME + ($providerNumber == null ? 43 : $providerNumber.hashCode());
    final java.lang.Object $patientStatus = this.getPatientStatus();
    result = result * PRIME + ($patientStatus == null ? 43 : $patientStatus.hashCode());
    final java.lang.Object $cmgCode = this.getCmgCode();
    result = result * PRIME + ($cmgCode == null ? 43 : $cmgCode.hashCode());
    final java.lang.Object $specialPayIndicator = this.getSpecialPayIndicator();
    result = result * PRIME + ($specialPayIndicator == null ? 43 : $specialPayIndicator.hashCode());
    final java.lang.Object $dischargeDate = this.getDischargeDate();
    result = result * PRIME + ($dischargeDate == null ? 43 : $dischargeDate.hashCode());
    final java.lang.Object $coveredCharges = this.getCoveredCharges();
    result = result * PRIME + ($coveredCharges == null ? 43 : $coveredCharges.hashCode());
    return result;
  }

  @java.lang.Override
  @java.lang.SuppressWarnings("all")
  @lombok.Generated
  public java.lang.String toString() {
    return "BillingRecord(npi=" + this.getNpi() + ", providerNumber=" + this.getProviderNumber() + ", patientStatus=" + this.getPatientStatus() + ", cmgCode=" + this.getCmgCode() + ", lengthOfStay=" + this.getLengthOfStay() + ", coveredDays=" + this.getCoveredDays() + ", lifetimeReserveDays=" + this.getLifetimeReserveDays() + ", specialPayIndicator=" + this.getSpecialPayIndicator() + ", dischargeDate=" + this.getDischargeDate() + ", coveredCharges=" + this.getCoveredCharges() + ")";
  }

  @java.lang.SuppressWarnings("all")
  @lombok.Generated
  public BillingRecord() {
  }
}
