// Generated by delombok at Mon Oct 04 14:27:44 EDT 2021
package gov.cms.fiss.pricers.irf.core;

import com.fasterxml.jackson.annotation.JsonIgnore;
import gov.cms.fiss.pricers.common.application.rules.CalculationContext;
import gov.cms.fiss.pricers.common.util.LocalDateUtils;
import gov.cms.fiss.pricers.irf.api.BillingRecord;
import gov.cms.fiss.pricers.irf.api.IrfInput;
import gov.cms.fiss.pricers.irf.api.IrfOutput;
import gov.cms.fiss.pricers.irf.api.IrfResult;
import gov.cms.fiss.pricers.irf.api.ProspectivePaymentRecord;
import gov.cms.fiss.pricers.irf.api.ProviderRecord;
import gov.cms.fiss.pricers.irf.core.tables.CbsaTableEntry;
import gov.cms.fiss.pricers.irf.core.tables.CmgTableEntry;
import gov.cms.fiss.pricers.irf.core.tables.DataTables;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.time.LocalDate;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashSet;
import java.util.Set;
import org.apache.commons.lang3.StringUtils;

public abstract class IrfPricerContext extends CalculationContext<IrfInput, IrfOutput> {
  // Constants related to Length of Stay / calendar
  public static final BigDecimal HALF_DAY_LENGTH_OF_STAY = new BigDecimal("0.5");
  public static final int THREE_DAY_LENGTH_OF_STAY = 3;
  public static final int FOURTEEN_DAY_LENGTH_OF_STAY = 14;
  public static final int SIXTEEN_DAY_LENGTH_OF_STAY = 16;
  // Calculation Values
  public static final BigDecimal TWENTY_FIVE_PERCENT = new BigDecimal("0.25");
  public static final BigDecimal EIGHTY_PERCENT = new BigDecimal("0.80");
  public static final String YES_INDICATOR = "Y";
  // Case Mix Groups
  public static final String CMG_5001 = "5001";
  public static final String CMG_9999 = "9999";
  public static final String CMG_2103 = "2103";
  public static final String CMG_A5001 = "A5001";
  public static final String CMG_A5101 = "A5101";
  public static final String CMG_A5102 = "A5102";
  public static final String CMG_A5103 = "A5103";
  public static final String CMG_A5104 = "A5104";
  public static final BigDecimal FEDERAL_RATE_FULL = BigDecimal.ONE;
  public static final BigDecimal FEDERAL_RATE_TWO_THIRDS = new BigDecimal("0.6667");
  public static final BigDecimal FACILITY_RATE_0 = BigDecimal.ZERO;
  public static final BigDecimal FACILITY_RATE_ONE_THIRD = new BigDecimal("0.3333");
  public static final BigDecimal RURAL_ADJUSTMENT_NONE = BigDecimal.ONE;
  public static final BigDecimal TRANSFER_PERCENT_FULL = BigDecimal.ONE;
  // from ProvNewHold
  public static final String SPECIAL_PAY_INDICATOR = "1";
  public static final String PROV_WAIVER_STATE = "Y";
  public static final String PROV_BLEND_INDICATOR_3 = "3";
  public static final String PROV_BLEND_INDICATOR_4 = "4";
  public static final String PROV_CBSA_HOSPITAL_QUALITY_INDICATOR_1 = "1";
  private static final int DEFAULT_MAX_LIFETIME_RESERVE_DAYS = 60;
  // Per-Year constants
  private static final BigDecimal DEFAULT_NATIONAL_LABOR_PERCENTAGE = new BigDecimal("0.69294");
  private static final BigDecimal DEFAULT_NATIONAL_NON_LABOR_PERCENTAGE = new BigDecimal("0.30706");
  private static final BigDecimal DEFAULT_NATIONAL_THRESHOLD_ADJUSTMENT = new BigDecimal("8848.00");
  private static final BigDecimal DEFAULT_BUDGET_NEUTRAL_CONVERSION_AMOUNT = new BigDecimal("15198.00");
  private static final BigDecimal DEFAULT_BUDGET_NEUTRAL_CONVERSION_AMOUNT2 = new BigDecimal("14901.00");
  private static final BigDecimal DEFAULT_TRANSITION_RURAL_ADJUSTMENT = BigDecimal.ZERO;
  private static final BigDecimal DEFAULT_RURAL_ADJUSTMENT = new BigDecimal("1.1490");
  private static final BigDecimal DEFAULT_CALC_LOW_INCOME_PERCENTAGE = new BigDecimal("0.3177");
  private static final BigDecimal DEFAULT_CALC_TEACH = new BigDecimal("1.0163");
  private static final Set<String> ORTHOPEDIC_CMG_PREFIXES = new HashSet<>(Arrays.asList("07", "08", "09"));
  private static final Set<String> URBAN_TO_RURAL_PROVIDERS = new HashSet<>(Arrays.asList("013027", "08T009", "10T249", "14T011", "193044", "193079", "193096", "19T003", "19T144", "19T304", "28T023", "33T157", "34T015", "34T131", "393047", "39T151", "42T036", "42T067", "49T018", "50T002"));
  // Patient Status values
  private static final String PATIENT_STATUS_EXPIRED = "20";
  // Note: These codes were taken from 88 codes in the original COBOL
  private static final Set<String> PATIENT_STATUS_TRANSFERRED = new HashSet<>(Arrays.asList("02", "03", "61", "62", "63", "64", "82", "83", "89", "90", "91", "92"));
  // State variables
  // Lookup content
  @JsonIgnore
  protected final DataTables dataTables;
  /**
   * Corresponds to {@code WAGE-NEW-INDEX-RECORD-CBSA} from the COBOL source.
   */
  private CbsaTableEntry cbsaWageIndex;
  /**
   * Corresponds to {@code H-WK-DSH PIC 9(01)V9(04)} from the COBOL source.
   */
  private BigDecimal disproportionateShareAdjustment;
  /**
   * Corresponds to {@code H-TEACH-PCT PIC 9(01)V9(04)} from the COBOL source.
   */
  private BigDecimal teachPercent;
  /**
   * Corresponds to {@code P-NEW-FED-PPS-BLEND-IND PIC X} from the COBOL source.
   */
  private String ppsBlendIndicator;

  protected IrfPricerContext(IrfInput input, IrfOutput output, DataTables dataTables) {
    super(input, output);
    this.dataTables = dataTables;
    initializeOutput();
  }

  protected void initializeOutput() {
    getOutput().setRequestId(getInput().getRequestId());
    // From 3000-CALC-PAYMENT
    getProspectivePaymentRecord().setTransferPercent(IrfPricerContext.TRANSFER_PERCENT_FULL.setScale(4, RoundingMode.HALF_UP));
  }

  // Calculation data
  public static boolean isOrthopedicCmg(String cmgCode) {
    return ORTHOPEDIC_CMG_PREFIXES.contains(StringUtils.mid(cmgCode, 1, 2));
  }

  public static Set<String> getUrbanToRuralProviders() {
    return Collections.unmodifiableSet(URBAN_TO_RURAL_PROVIDERS);
  }

  @JsonIgnore
  public BillingRecord getBillingRecord() {
    return getInput().getBillingRecord();
  }

  @JsonIgnore
  public ProviderRecord getProviderRecord() {
    return getInput().getProviderRecord();
  }

  @JsonIgnore
  public ProspectivePaymentRecord getProspectivePaymentRecord() {
    return getOutput().getRecord();
  }

  @JsonIgnore
  public ResultCode getResultCode() {
    return getProspectivePaymentRecord().getResult().getResultCode();
  }

  @JsonIgnore
  public LocalDate getBillingDischargeDate() {
    return LocalDateUtils.fromYearFormat(getBillingRecord().getDischargeDate());
  }

  /**
   * Determines if the given code matches the current output record code.
   */
  public boolean matchesReturnCode(ResultCode expectedCode) {
    return expectedCode.equals(ResultCode.fromCode(getResultCode().getCode()));
  }

  /**
   * Determines if the given CBSA is rural.
   */
  public boolean isRuralCbsa(String cbsa) {
    return cbsa.trim().matches("\\d{2}");
  }

  // Manage state
  @JsonIgnore
  public LocalDate getProviderEffectiveDate() {
    return LocalDateUtils.fromYearFormat(getProviderRecord().getEffectiveDate());
  }

  @JsonIgnore
  public LocalDate getProviderFiscalYearBegin() {
    return LocalDateUtils.fromYearFormat(getProviderRecord().getFiscalYearBeginDate());
  }

  public void applyResultCode(ResultCode resultCode) {
    getProspectivePaymentRecord().setResult(IrfResult.fromResultCode(resultCode));
  }

  // Lookup content
  public void applyResultCode(ResultCode resultCode, String explanation) {
    applyResultCode(resultCode);
    getProspectivePaymentRecord().getResult().setExplanation(explanation);
  }

  /**
   * Resets the prospective payment record back to initial/default values.
   */
  public void zeroPaymentRecord() {
    final ProspectivePaymentRecord zeroRecord = new ProspectivePaymentRecord();
    zeroRecord.setResult(getProspectivePaymentRecord().getResult());
    getOutput().setRecord(zeroRecord);
  }

  // Per-year value retrieval
  public CbsaTableEntry getCbsaWageIndex(String cbsa, String providedEffectiveDate) {
    return dataTables.getCbsaWageIndex(cbsa, providedEffectiveDate);
  }

  public CmgTableEntry getCmgEntry(String cmgCode) {
    return dataTables.getCmgEntry(cmgCode);
  }

  public BigDecimal getBudgetNeutralConversionAmount() {
    return DEFAULT_BUDGET_NEUTRAL_CONVERSION_AMOUNT;
  }

  public BigDecimal getBudgetNeutralConversionAmount2() {
    return DEFAULT_BUDGET_NEUTRAL_CONVERSION_AMOUNT2;
  }

  public BigDecimal getCalcLowIncomePercentage() {
    return DEFAULT_CALC_LOW_INCOME_PERCENTAGE;
  }

  public BigDecimal getCalcTeach() {
    return DEFAULT_CALC_TEACH;
  }

  /**
   * Corresponds to {@code PPS-CALC-VERS-CD PIC X(05)} from the COBOL source.
   */
  public abstract String getCalcVersion();

  public abstract LocalDate getFiscalYearEnd();

  public abstract LocalDate getFiscalYearStart();

  public int getMaxLifetimeReserveDays() {
    return DEFAULT_MAX_LIFETIME_RESERVE_DAYS;
  }

  /**
   * Corresponds to {@code PPS-NAT-LABOR-PCT PIC 9(01)V9(05)} from the COBOL source.
   */
  public BigDecimal getNationalLaborPercentage() {
    return DEFAULT_NATIONAL_LABOR_PERCENTAGE;
  }

  /**
   * Corresponds to {@code PPS-NAT-NONLABOR-PCT PIC 9(01)V9(05)} from the COBOL source.
   */
  public BigDecimal getNationalNonLaborPercentage() {
    return DEFAULT_NATIONAL_NON_LABOR_PERCENTAGE;
  }

  /**
   * Corresponds to {@code PPS-NAT-THRESHOLD-ADJ PIC 9(05)V9(02)} from the COBOL source.
   */
  public BigDecimal getNationalThresholdAdjustment() {
    return DEFAULT_NATIONAL_THRESHOLD_ADJUSTMENT;
  }

  public BigDecimal getTransitionRuralAdjustment() {
    return DEFAULT_TRANSITION_RURAL_ADJUSTMENT;
  }

  public BigDecimal getRuralAdjustment() {
    return DEFAULT_RURAL_ADJUSTMENT;
  }

  // Business methods
  // Business methods
  public BigDecimal getWageIndexPercentAdjustment() {
    return BigDecimal.ZERO;
  }

  public BigDecimal getWageIndexPercentReductionCap() {
    return BigDecimal.ZERO;
  }

  // Billing record business methods
  public boolean hasPenalties() {
    return StringUtils.equalsAny(getBillingRecord().getSpecialPayIndicator(), "2", "3");
  }

  // It feels like this should maybe be just 'isOutlier', and look for "2", but I don't know at the
  // moment what these indicators mean so I'm exactly replicating what was here before.
  public boolean isNotOutlier() {
    return StringUtils.equalsAny(getBillingRecord().getSpecialPayIndicator(), "1", "3");
  }

  public boolean isPatientExpired() {
    return getBillingRecord().getPatientStatus().equals(PATIENT_STATUS_EXPIRED);
  }

  public boolean isPatientTransferred() {
    return PATIENT_STATUS_TRANSFERRED.contains(getBillingRecord().getPatientStatus());
  }

  public boolean isTransferCase() {
    return PATIENT_STATUS_TRANSFERRED.contains(getBillingRecord().getPatientStatus()) && getProspectivePaymentRecord().getLengthOfStay() < getProspectivePaymentRecord().getAverageLengthOfStay();
  }

  /**
   * Corresponds to {@code WAGE-NEW-INDEX-RECORD-CBSA} from the COBOL source.
   */
  @java.lang.SuppressWarnings("all")
  @lombok.Generated
  public CbsaTableEntry getCbsaWageIndex() {
    return this.cbsaWageIndex;
  }

  /**
   * Corresponds to {@code WAGE-NEW-INDEX-RECORD-CBSA} from the COBOL source.
   */
  @java.lang.SuppressWarnings("all")
  @lombok.Generated
  public void setCbsaWageIndex(final CbsaTableEntry cbsaWageIndex) {
    this.cbsaWageIndex = cbsaWageIndex;
  }

  /**
   * Corresponds to {@code H-WK-DSH PIC 9(01)V9(04)} from the COBOL source.
   */
  @java.lang.SuppressWarnings("all")
  @lombok.Generated
  public BigDecimal getDisproportionateShareAdjustment() {
    return this.disproportionateShareAdjustment;
  }

  /**
   * Corresponds to {@code H-WK-DSH PIC 9(01)V9(04)} from the COBOL source.
   */
  @java.lang.SuppressWarnings("all")
  @lombok.Generated
  public void setDisproportionateShareAdjustment(final BigDecimal disproportionateShareAdjustment) {
    this.disproportionateShareAdjustment = disproportionateShareAdjustment;
  }

  /**
   * Corresponds to {@code H-TEACH-PCT PIC 9(01)V9(04)} from the COBOL source.
   */
  @java.lang.SuppressWarnings("all")
  @lombok.Generated
  public BigDecimal getTeachPercent() {
    return this.teachPercent;
  }

  /**
   * Corresponds to {@code H-TEACH-PCT PIC 9(01)V9(04)} from the COBOL source.
   */
  @java.lang.SuppressWarnings("all")
  @lombok.Generated
  public void setTeachPercent(final BigDecimal teachPercent) {
    this.teachPercent = teachPercent;
  }

  /**
   * Corresponds to {@code P-NEW-FED-PPS-BLEND-IND PIC X} from the COBOL source.
   */
  @java.lang.SuppressWarnings("all")
  @lombok.Generated
  public String getPpsBlendIndicator() {
    return this.ppsBlendIndicator;
  }

  /**
   * Corresponds to {@code P-NEW-FED-PPS-BLEND-IND PIC X} from the COBOL source.
   */
  @java.lang.SuppressWarnings("all")
  @lombok.Generated
  public void setPpsBlendIndicator(final String ppsBlendIndicator) {
    this.ppsBlendIndicator = ppsBlendIndicator;
  }
}
