package gov.cms.fiss.pricers.irf.core.rules;

import gov.cms.fiss.pricers.common.application.rules.CalculationRule;
import gov.cms.fiss.pricers.irf.api.IrfInput;
import gov.cms.fiss.pricers.irf.api.IrfOutput;
import gov.cms.fiss.pricers.irf.api.ProspectivePaymentRecord;
import gov.cms.fiss.pricers.irf.core.IrfPricerContext;
import gov.cms.fiss.pricers.irf.core.ResultCode;
import java.math.RoundingMode;

/**
 * Calculates the low income payment (LIP) amount.
 *
 * <p>Converted from {@code 3500-CONTINUE-CALC} in the COBOL code.
 */
public class CalculateLowIncomePaymentAmount
    implements CalculationRule<IrfInput, IrfOutput, IrfPricerContext> {

  @Override
  public boolean shouldExecute(IrfPricerContext calculationContext) {
    return calculationContext.matchesReturnCode(ResultCode.OK_00);
  }

  @Override
  public void priceClaim(IrfPricerContext calculationContext) {
    final ProspectivePaymentRecord prospectivePaymentRecord =
        calculationContext.getProspectivePaymentRecord();

    // ** RULE: THE PPS LOW INCOME PATIENT PAYMENT AMOUNT IS COMPUTED AS
    // ** THE PPS FEDERAL PAY AMOUNT TIMES THE PPS LOW INCOME PATIENT
    // ** PERCENTAGE) ROUNDED

    // COMPUTE PPS-LIP-PAY-AMT ROUNDED =
    //         (PPS-FED-PAY-AMT * PPS-LIP-PCT).
    prospectivePaymentRecord.setLowIncomePaymentAmount(
        prospectivePaymentRecord
            .getFederalPaymentAmount()
            .multiply(prospectivePaymentRecord.getLowIncomePaymentPercent())
            .setScale(2, RoundingMode.HALF_UP));
  }
}
