package gov.cms.fiss.pricers.irf.core.rules;

import gov.cms.fiss.pricers.common.application.rules.CalculationRule;
import gov.cms.fiss.pricers.common.util.BigDecimalUtils;
import gov.cms.fiss.pricers.irf.api.IrfInput;
import gov.cms.fiss.pricers.irf.api.IrfOutput;
import gov.cms.fiss.pricers.irf.api.ProspectivePaymentRecord;
import gov.cms.fiss.pricers.irf.core.IrfPricerContext;
import gov.cms.fiss.pricers.irf.core.ResultCode;

/**
 * Calculates the standard payment.
 *
 * <p>Converted from {@code 3500-CONTINUE-CALC} in the COBOL code.
 */
public class CalculateStandardPayment
    implements CalculationRule<IrfInput, IrfOutput, IrfPricerContext> {

  @Override
  public boolean shouldExecute(IrfPricerContext calculationContext) {
    return calculationContext.matchesReturnCode(ResultCode.OK_00);
  }

  @Override
  public void priceClaim(IrfPricerContext calculationContext) {
    final ProspectivePaymentRecord prospectivePaymentRecord =
        calculationContext.getProspectivePaymentRecord();

    // COMPUTE PPS-STANDARD-PAY-AMT =
    //             (PPS-TRANSFER-PCT * PPS-RELATIVE-WGT
    //                       * PPS-BDGT-NEUT-CONV-AMT).
    prospectivePaymentRecord.setStandardPaymentAmount(
        BigDecimalUtils.truncateDecimals(
            prospectivePaymentRecord
                .getTransferPercent()
                .multiply(prospectivePaymentRecord.getRelativeWeight())
                .multiply(prospectivePaymentRecord.getBudgetNeutralConversionAmount())));
  }
}
