package gov.cms.fiss.pricers.irf.core.rules;

import gov.cms.fiss.pricers.common.application.rules.CalculationRule;
import gov.cms.fiss.pricers.irf.api.BillingRecord;
import gov.cms.fiss.pricers.irf.api.IrfInput;
import gov.cms.fiss.pricers.irf.api.IrfOutput;
import gov.cms.fiss.pricers.irf.api.ProspectivePaymentRecord;
import gov.cms.fiss.pricers.irf.core.IrfPricerContext;
import gov.cms.fiss.pricers.irf.core.ResultCode;
import gov.cms.fiss.pricers.irf.core.tables.CbsaTableEntry;
import java.math.RoundingMode;

public class DetermineRuralAdjustment
    implements CalculationRule<IrfInput, IrfOutput, IrfPricerContext> {

  @Override
  public boolean shouldExecute(IrfPricerContext calculationContext) {
    return calculationContext.matchesReturnCode(ResultCode.OK_00);
  }

  @Override
  public void priceClaim(IrfPricerContext calculationContext) {
    final BillingRecord billingRecord = calculationContext.getBillingRecord();
    final ProspectivePaymentRecord prospectivePaymentRecord =
        calculationContext.getProspectivePaymentRecord();
    final CbsaTableEntry billingCbsa = calculationContext.getCbsaWageIndex();

    // MOVE 1.0000          TO PPS-RURAL-ADJUSTMENT
    prospectivePaymentRecord.setRuralAdjustment(
        IrfPricerContext.RURAL_ADJUSTMENT_NONE.setScale(4, RoundingMode.HALF_UP));

    // IF VALID-PROV-NUM
    if (IrfPricerContext.getUrbanToRuralProviders().contains(billingRecord.getProviderNumber())) {
      // MOVE 1.0497       TO PPS-RURAL-ADJUSTMENT
      prospectivePaymentRecord.setRuralAdjustment(
          calculationContext.getTransitionRuralAdjustment().setScale(4, RoundingMode.HALF_UP));
    } else if (calculationContext.isRuralCbsa(billingCbsa.getCbsa())) {
      // ELSE
      //    IF W-NEW-CBSA (1:3) = '   '
      //       MOVE 1.1490 TO PPS-RURAL-ADJUSTMENT
      prospectivePaymentRecord.setRuralAdjustment(
          calculationContext.getRuralAdjustment().setScale(4, RoundingMode.HALF_UP));
    } else {
      //    ELSE
      //       MOVE 1.0000 TO PPS-RURAL-ADJUSTMENT.
      prospectivePaymentRecord.setRuralAdjustment(
          IrfPricerContext.RURAL_ADJUSTMENT_NONE.setScale(4, RoundingMode.HALF_UP));
    }
  }
}
