package gov.cms.fiss.pricers.irf.core.rules.bill_review;

import gov.cms.fiss.pricers.common.application.rules.CalculationRule;
import gov.cms.fiss.pricers.irf.api.BillingRecord;
import gov.cms.fiss.pricers.irf.api.IrfInput;
import gov.cms.fiss.pricers.irf.api.IrfOutput;
import gov.cms.fiss.pricers.irf.api.ProspectivePaymentRecord;
import gov.cms.fiss.pricers.irf.core.IrfPricerContext;
import gov.cms.fiss.pricers.irf.core.ResultCode;
import java.math.RoundingMode;

/**
 * Initializes national values.
 *
 * <p>Converted from {@code 1000-EDIT-THE-BILL-INFO} in the COBOL code.
 */
public class InitializeNationalValues
    implements CalculationRule<IrfInput, IrfOutput, IrfPricerContext> {

  @Override
  public boolean shouldExecute(IrfPricerContext calculationContext) {
    return calculationContext.matchesReturnCode(ResultCode.OK_00);
  }

  @Override
  public void priceClaim(IrfPricerContext calculationContext) {
    final BillingRecord billingRecord = calculationContext.getBillingRecord();
    final ProspectivePaymentRecord prospectivePaymentRecord =
        calculationContext.getProspectivePaymentRecord();

    // MOVE B-CMG-CODE TO PPS-SUBM-CMG-CODE.
    prospectivePaymentRecord.setSubmittedCmgCode(billingRecord.getCmgCode());

    // MOVE .72700 TO PPS-NAT-LABOR-PCT.
    prospectivePaymentRecord.setNationalLaborPercent(
        calculationContext.getNationalLaborPercentage().setScale(5, RoundingMode.HALF_UP));

    // MOVE .27300 TO PPS-NAT-NONLABOR-PCT.
    prospectivePaymentRecord.setNationalNonLaborPercent(
        calculationContext.getNationalNonLaborPercentage().setScale(5, RoundingMode.HALF_UP));

    // MOVE  9300  TO PPS-NAT-THRESHOLD-ADJ.
    prospectivePaymentRecord.setNationalThresholdAdjustment(
        calculationContext.getNationalThresholdAdjustment().setScale(2, RoundingMode.HALF_UP));
  }
}
