package gov.cms.fiss.pricers.irf.core.rules.bill_review;

import gov.cms.fiss.pricers.common.application.rules.CalculationRule;
import gov.cms.fiss.pricers.common.util.LocalDateUtils;
import gov.cms.fiss.pricers.irf.api.IrfInput;
import gov.cms.fiss.pricers.irf.api.IrfOutput;
import gov.cms.fiss.pricers.irf.api.ProviderRecord;
import gov.cms.fiss.pricers.irf.core.IrfPricerContext;
import gov.cms.fiss.pricers.irf.core.ResultCode;

/**
 * Validate provider status.
 *
 * <pre>
 * ** RULE: A CLAIM WILL NOT BE PAID IF THE CLAIM'S DISCHARGE DATE
 * ** IS ON OR AFTER THE DATE WHEN A PROVIDER TERMINATED COVERAGE
 * ** REQUIREMENT: THE SYSTEM MUST RETURN AN ERROR CODE (51) WHEN
 * ** THE BILL'S DISCHARGE DATE IS ON/AFTER THE PROVIDER TERMINATION
 * ** DATE
 * </pre>
 *
 * Converted from {@code 1000-EDIT-THE-BILL-INFO} in the COBOL code.
 */
public class ValidateProviderStatus
    implements CalculationRule<IrfInput, IrfOutput, IrfPricerContext> {

  @Override
  public boolean shouldExecute(IrfPricerContext calculationContext) {
    return calculationContext.matchesReturnCode(ResultCode.OK_00);
  }

  @Override
  public void priceClaim(IrfPricerContext calculationContext) {
    final ProviderRecord providerRecord = calculationContext.getProviderRecord();

    // IF P-NEW-TERMINATION-DATE > 00000000
    //    IF B-DISCHARGE-DATE >= P-NEW-TERMINATION-DATE
    if (providerRecord.getTerminationDate().compareTo("00000000") > 0
        && LocalDateUtils.isAfterOrEqual(
            calculationContext.getBillingDischargeDate(),
            LocalDateUtils.fromYearFormat(providerRecord.getTerminationDate()))) {
      //    MOVE 51 TO PPS-RTC.
      calculationContext.applyResultCode(ResultCode.PROV_REC_TERMINATED_51);
    }
  }
}
