package gov.cms.fiss.pricers.irf.core.rules.finalize_payment;

import gov.cms.fiss.pricers.common.application.rules.CalculationRule;
import gov.cms.fiss.pricers.irf.api.IrfInput;
import gov.cms.fiss.pricers.irf.api.IrfOutput;
import gov.cms.fiss.pricers.irf.api.ProspectivePaymentRecord;
import gov.cms.fiss.pricers.irf.core.IrfPricerContext;
import java.math.RoundingMode;

/**
 * Calculates total payment.
 *
 * <p>Converted from {@code 5000-FINAL-PAYMENTS} in the COBOL code.
 */
public class CalculateTotalPayment
    implements CalculationRule<IrfInput, IrfOutput, IrfPricerContext> {

  @Override
  public void priceClaim(IrfPricerContext calculationContext) {
    final ProspectivePaymentRecord prospectivePaymentRecord =
        calculationContext.getProspectivePaymentRecord();

    // ** RULE: THE PPS TOTAL PAY AMOUNT IS COMPUTED AS THE PPS FEDERAL
    // ** PAY AMOUNT PLUS THE PPS FACILITY SPECIFIC PAY AMOUNT PLUS THE
    // ** PPS OUTLIER PAY AMOUNT PLUS THE PPS LOW INCOME PATIENT PAY
    // ** AMOUNT + PPS TEACH PAY AMOUNT

    // COMPUTE PPS-TOTAL-PAY-AMT ROUNDED =
    //         (PPS-FED-PAY-AMT + PPS-FAC-SPEC-PAY-AMT
    //          + PPS-OUTLIER-PAY-AMT + PPS-LIP-PAY-AMT +
    //          PPS-TEACH-PAY-AMT).
    prospectivePaymentRecord.setTotalPaymentAmount(
        prospectivePaymentRecord
            .getFederalPaymentAmount()
            .add(prospectivePaymentRecord.getFacilitySpecificPaymentAmount())
            .add(prospectivePaymentRecord.getOutlierPaymentAmount())
            .add(prospectivePaymentRecord.getLowIncomePaymentAmount())
            .add(prospectivePaymentRecord.getTeachPaymentAmount())
            .setScale(2, RoundingMode.HALF_UP));
  }
}
