package gov.cms.fiss.pricers.irf.core.rules.finalize_payment;

import gov.cms.fiss.pricers.common.application.rules.CalculationRule;
import gov.cms.fiss.pricers.irf.api.IrfInput;
import gov.cms.fiss.pricers.irf.api.IrfOutput;
import gov.cms.fiss.pricers.irf.api.ProspectivePaymentRecord;
import gov.cms.fiss.pricers.irf.core.IrfPricerContext;
import java.math.BigDecimal;
import java.math.RoundingMode;

/**
 * Finalizes outlier payment.
 *
 * <p>Converted from {@code 5000-FINAL-PAYMENTS} in the COBOL code.
 */
// TODO: DDS to evaluate renaming since this is not finalizing the outlier payment
public class FinalizeOutlierPayment
    implements CalculationRule<IrfInput, IrfOutput, IrfPricerContext> {

  @Override
  public void priceClaim(IrfPricerContext calculationContext) {
    final ProspectivePaymentRecord prospectivePaymentRecord =
        calculationContext.getProspectivePaymentRecord();

    // ** RULE: CLAIMS WITH NO SPECIAL OUTLIER PAYMENTS ARE ASSIGNED
    // ** ZERO DOLLARS FOR THE OUTLIER PAYMENT

    // IF B-SPEC-PAY-IND = '1' OR '3'
    if (calculationContext.isNotOutlier()) {
      // MOVE ZEROES TO PPS-OUTLIER-PAY-AMT.
      prospectivePaymentRecord.setOutlierPaymentAmount(
          BigDecimal.ZERO.setScale(2, RoundingMode.HALF_UP));
    }
  }
}
